#!/bin/bash
#
# Usage: install_daemon.sh <imagefile>
#
# VERSION=25
# CHANGES="Checking for Major < 17 since it is not possible to downgrade to those versions."

# constants #
BEROCONF=/usr/fallback/beroconf
PIDFILE=/var/run/install_daemon.pid

file=${1}
image_dir=/tmp/images

# functions #
function log {

	prefix=${1:common}
	logfile=/usr/conf/permlog/install-${prefix}.log
	pipe=/tmp/install_pipe

	if [ -p ${pipe} ]; then
		echo "[install_${prefix}] $(date): ${2}" &>${pipe}
	fi

	echo "[install_${prefix}] $(date): ${2}" >> ${logfile}
}

function fix_old_image_dir {
	mount -o remount,rw /
	if [ -d /mnt/images ]; then
		rm -rf /mnt/images
	fi
	ln -s /tmp /mnt/images
	mount -o remount,ro /
}

function tmp_cleanup {
	for i in "VERSION content.txt compare-content.txt"; do
		rm -f ${i}
	done
}

# main #
if [ ! -d ${image_dir} ]; then
	mkdir -p ${image_dir}
fi

if [ -z "${file}" ] ; then
	echo "ERROR: No filename given, leaving."
	rm -f ${PIDFILE}
	exit 1
fi

if [ ! -f "${image_dir}/${file}" ] ; then
	echo "ERROR: File ${image_dir}/${file} not found, leaving."
	rm -f ${PIDFILE}
	exit 1
fi

if [ ! -L /mnt/images ]; then
	fix_old_image_dir
fi

cd /tmp && tar zxf ${image_dir}/${file} VERSION
. /tmp/VERSION

# check if TYPE-field is sane.
case "${TYPE}" in
	fpga|firmware|kernel|rootfs|appfs|userappfs)
		;;
	*)
		log "ERROR: This package does not contain a valid type, leaving."
		rm -f ${PIDFILE}
		exit 1
		;;
esac

# remove previous log-file
rm -f /usr/conf/permlog/install-${TYPE}.log

log "${TYPE}" "Installing '${file}'."
case "${TYPE}" in
	fpga)
		# Put on a LED-Show for the customers!
		if [ -f /sys/class/beronet/gateway/led_program ]; then
			echo -n 3 > /sys/class/beronet/gateway/led_program
		fi

		log "${TYPE}" "Unpacking FPGA-Image."
		cd /tmp && tar zxf ${image_dir}/${file} fpga.img
		if [ ! -f /tmp/fpga.img ] ; then
			log "${TYPE}" "ERROR: Could not unpack the FPGA-Image, leaving."
			exit 1
		fi

		# retrieve Version-Information
		VERSION_INSTALLED=$(expr match "$(cat /sys/class/beronet/fpga/version)" "\([0-9]*\)")
		VERSION_TOINSTALL="$(expr match "$(cat /tmp/VERSION)" ".*FPGA=\(.*\)")"

		# tell the user about it
		log "${TYPE}" "Installing FPGA-Image. This process may take 3 minutes."
		log "${TYPE}" "Do NOT turn off your beroNet VoIP Gateway!"
		log "${TYPE}" "Updating FPGA-Image from Version ${VERSION_INSTALLED} to ${VERSION_TOINSTALL}."

		# log update-operations
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log

		# activate write-mode on fpga-eeprom
		echo 1 > /sys/class/beronet/fpga/eeprom
		if [ "$(cat /sys/class/beronet/fpga/eeprom)" -ne "1" ]; then
			log "${TYPE}" "ERROR: Could not enable write-access to FPGA-eeprom, leaving."
			exit 1
		fi

		# prepare the next reboot
		${BEROCONF} set root boot_fwupdate 0
		echo "1" > /tmp/reboot_after_update

		# Perform the Update
		sleep 2
		dd if=/tmp/fpga.img of=/dev/bfpga bs=512k count=1
		;;
	firmware)
		# Put on a LED-Show for the customers!
		if [ -f /sys/class/beronet/gateway/led_program ]; then
			echo -n 3 > /sys/class/beronet/gateway/led_program
		fi

		log "${TYPE}" "Unpacking Firmware."
		cd /tmp && tar zxf ${image_dir}/${file} firmware.axf
		if [ ! -f /tmp/firmware.axf ]; then
			log "${TYPE}" "ERROR: Could not unpack the Firmware-Image, leaving."
			exit 1
		fi

		# retrieve Version-Information
		VERSION_INSTALLED="unknown"
		if [ -f /usr/conf/VERSION.firmware ]; then
			VERSION_INSTALLED="$(expr match "$(cat /usr/conf/VERSION.firmware)" ".*FIRMWARE=\(.*\)")"
		fi
		VERSION_TOINSTALL="$(expr match "$(cat /tmp/VERSION)" ".*FIRMWARE=\(.*\)")"

		log "${TYPE}" "Installing Firmware-Image. This process may take 3 minutes."
		log "${TYPE}" "Do NOT turn off your beroNet VoIP Gateway!"
		log "${TYPE}" "Updating Firmware-Image from Version ${VERSION_INSTALLED} to ${VERSION_TOINSTALL}."

		# prepare the next reboot
		${BEROCONF} set root boot_fwupdate 0
		echo "1" > /tmp/reboot_after_update

		# log update-operations
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log
		mv /tmp/VERSION /usr/conf/VERSION.firmware

		# Perform the Update
		sleep 2
		dd if=/tmp/firmware.axf of=/dev/mtdblock3
	;;
	kernel)
		# Put on a LED-Show for the customers!
		if [ -f /sys/class/beronet/gateway/led_program ]; then
			echo -n 3 > /sys/class/beronet/gateway/led_program
		fi

		log "${TYPE}" "Unpacking Kernel-Image."
		cd /tmp && tar zxf ${image_dir}/${file} zImage
		if [ ! -f /tmp/zImage ]; then
			log "${TYPE}" "ERROR: Could not unpack the Kernel-Image, leaving."
			exit 1
		fi

		# Retrieve Version-Information
		VERSION_INSTALLED="unknown"
		if [ -f /usr/conf/VERSION.${TYPE} ]; then
			VERSION_INSTALLED="$($(expr match "$(cat /usr/conf/VERSION.kernel)" ".*KERNEL=\(.*\)"))"
		fi
		VERSION_TOINSTALL="$($(expr match "$(cat /tmp/VERSION)" ".*KERNEL=\(.*\)"))"

		log "${TYPE}" "Installing Kernel-Image. This process may take 3 minutes."
		log "${TYPE}" "Do NOT turn off your beroNet VoIP Gateway!"
		log "${TYPE}" "Updating Kernel-Image from Version ${VERSION_INSTALLED} to ${VERSION_TOINSTALL}."

		# log update-operations
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log
		mv /tmp/VERSION /usr/conf/VERSION.kernel

		# prepare the next reboot
		${BEROCONF} set root boot_fwupdate 0
		echo "1" > /tmp/reboot_after_update

		sleep 2
		dd if=/tmp/zImage of=/dev/mtdblock4
	;;
	rootfs)
		# Put on a LED-Show for the customers!
		if [ -f /sys/class/beronet/gateway/led_program ]; then
			echo -n 3 > /sys/class/beronet/gateway/led_program
		fi

		VERSION_INSTALLED="unknown"
		if [ -f /VERSION ]; then
			VERSION_INSTALLED=$(expr match "$(cat /VERSION)" ".*ROOTFS=\(.*\)")
		fi
		VERSION_TOINSTALL=$(expr match "$(cat /tmp/VERSION)" ".*ROOTFS=\(.*\)")

		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log

		if [ -f /etc/ssh_host_key ]; then
			log "${TYPE}" "Saving SSH-Leys to /usr/conf."
			[ ! -d /usr/conf/ssh ] && mkdir /usr/conf/ssh
			cp -a /etc/ssh_host* /usr/conf/ssh/
		fi

		log "${TYPE}" "Stopping services."
		/etc/init.d/S90app stop
		/etc/init.d/S60httpd stop

		# prepare the next reboot
		${BEROCONF} set root boot_fwupdate 0

		log "${TYPE}" "Loading berofix module."
		insmod /usr/local/modules/berofix.ko

		log "${TYPE}" "Extracting RootFS-Image."
		rm -rf /tmp/newroot
		mkdir /tmp/newroot || exit 1
		cd /tmp/newroot
		tar zxf ${image_dir}/${file} root.jffs2 || exit 1

		log "${TYPE}" "Saving RootFS to flash memory."
		dd if=root.jffs2 of=/dev/mtdblock5

		log "${TYPE}" "rebooting"
		echo 1 > /sys/class/beronet/fpga/reset
		;;
	userappfs)
		/usr/sbin/userapp_pkg_install.sh ${file}
		rm -f ${PIDFILE}
		;;
	appfs)
		# Put on a LED-Show for the customers!
		if [ -f /sys/class/beronet/gateway/led_program ]; then
			echo -n 3 > /sys/class/beronet/gateway/led_program
		fi

		# downgrades to appfs older than 17 is no longer supported
		if [ -z "${APPFS}" ]; then
			log "${TYPE}" "Downgrading to a Firmware older than 17.01 is not supported."
			sleep 10
			tmp_cleanup
			rm -f ${PIDFILE}
			echo -n 2 > /sys/class/beronet/gateway/led_program
			exit 1
		elif [ ${APPFS} -lt 17 ]; then
		
			if [ ! -f /tmp/DOWNGRADED ]; then
				rm -f /tmp/DOWNGRADED
				log "${TYPE}" "Downgrading to a Firmware older than 17.01 is not supported."
				sleep 10
				tmp_cleanup
				rm -f ${PIDFILE}
				echo -n 2 > /sys/class/beronet/gateway/led_program
				exit 1
			fi
			log "${TYPE}" "Installing older Firmware. Basic downgrade was made, continuing with Firmware install."

		fi


		rm -rf /tmp/content.txt
                rm -rf /tmp/compare-content.txt

		log "${TYPE}" "Checking Versions of installed Kernel and Firmware."
		if [ -f /usr/conf/VERSION.kernel ]; then
			KERNEL=$(expr match "$(cat /usr/conf/VERSION.kernel)" ".*VERSION=\([0-9]*\)")
		else
			KERNEL=1
		fi
		if [ -f /sys/class/beronet/fpga/version ]; then
			FPGA=$(cat /sys/class/beronet/fpga/version)
		else
			FPGA=8
		fi
		if [ -f /usr/conf/VERSION.firmware ]; then
			FIRMWARE=$(expr match "$(cat /usr/conf/VERSION.firmware)" ".*VERSION=\([0-9]*\)")
		else
			FIRMWARE=1
		fi

		if [ -f /pkginfo/VERSION.rootfs ]; then
			ROOTFS=$(expr match "$(cat /pkginfo/VERSION.rootfs)" ".*PKG_VERSION=\([0-9]*\)")
		else
			ROOTFS=1
		fi

		if [ -z "${APPFS}" ]; then
			APPFS=1
		fi

		version_check_fail=0
		if [ "${NEED_KERNEL}" -gt "${KERNEL}" ]; then
			log "${TYPE}" "FAIL: Please update Kernel to version ${NEED_KERNEL} and try again!"
			version_check_fail=1
		fi
		if [ "${NEED_FPGA}" -gt "${FPGA}" ]; then
			log "${TYPE}" "FAIL: Please update FPGA to version ${NEED_FPGA} and try again!"
			version_check_fail=1
		fi
		if [ "${NEED_FIRMWARE}" -gt "${FIRMWARE}" ]; then
			log "${TYPE}" "FAIL: Please update Firmware to version ${NEED_FIRMWARE} and try again!"
			version_check_fail=1
		fi
		if [ "${NEED_ROOTFS}" -gt "${ROOTFS}" ]; then
			log "${TYPE}" "FAIL: Please update RootFS to version ${NEED_ROOTFS} and try again!"
			version_check_fail=1
		fi

		if [ "${version_check_fail}" = "1" ]; then
			sleep 10
			tmp_cleanup
			rm -f ${PIDFILE}
			echo -n 2 > /sys/class/beronet/gateway/led_program
			exit 1
		fi

		VERSION_INSTALLED="$(cat /usr/local/FILENAME)"

		if [ "$(${BEROCONF} get root boot_fwupdate)" != "1" ]; then
			/etc/init.d/S45cron stop > /dev/null

        	        log "${TYPE}" "Stopping UserAppFS-Services."
                	/etc/init.d/S91userapp stop > /dev/null

			log "${TYPE}" "Stopping AppFS-Services."
			/etc/init.d/S90app stop > /dev/null
		fi

		log "${TYPE}" "Remounting AppFS partition (rw)."
		mount -o remount,rw /usr/local || exit 1

		log "${TYPE}" "Executing pre_remove-Script."
		[ -x /usr/local/setup/pre_remove.sh ] && /usr/local/setup/pre_remove.sh ${APPFS}

		log "${TYPE}" "Removing old Version-File."
		[ -f /usr/local/FILENAME ] && rm -f /usr/local/FILENAME

		log "${TYPE}" "Cleaning up AppFS-Partition."
		cd /usr/local && ls | grep -v lost | xargs rm -rf

		log "${TYPE}" "Extracting new AppFS-Image."
		cd /usr/local && tar zxf ${image_dir}/${file} || exit 1

		log "${TYPE}" "Setting Permissions."
		chown -R root:root /usr/local/*

		log "${TYPE}" "Writing new Version-File."
		echo "${file}" > /usr/local/FILENAME

		sync; sleep 1; sync

                log "${TYPE}" "Removing uploaded AppFS-Image."
                rm -f ${image_dir}/${file}
		if [ -f /home/admin/${file} ]; then
			mount -oremount,rw /home/admin
			rm -f /home/admin/${file}
			mount -oremount,ro /home/admin
		fi

		log "${TYPE}" "Executing post_install-Script."
		[ -x /usr/local/setup/post_install.sh ] && /usr/local/setup/post_install.sh

		log "${TYPE}" "Remounting AppFS-Partition (ro)."
		cd / && mount -o remount,ro /usr/local

                sync

		${BEROCONF} set root boot_fwupdate 0
		${BEROCONF} set root app-image $(cat /usr/local/FILENAME)

		VERSION_TOINSTALL="$(cat /usr/local/FILENAME)"
		echo "[${TYPE}] $(date): ${VERSION_INSTALLED} => ${VERSION_TOINSTALL}" >> /usr/conf/update.log

		if [ ! -f /tmp/reboot_after_update ]; then
			# Remove integrity-marker.
			if [ -f /tmp/enable_recovery ]; then
				rm -f /tmp/enable_recovery
			fi

			# End LED-Show.
			if [ -f /sys/class/beronet/gateway/led_program ]; then
				echo -n 0 > /sys/class/beronet/gateway/led_program
			fi

			log "${TYPE}" "Restarting RootFS-Services."
			/sbin/rmmod /lib/modules/berofix-driver.ko
			/sbin/insmod /lib/modules/berofix-driver.ko
			/etc/init.d/S40network restart > /dev/null
			/etc/init.d/S41ncs restart > /dev/null
			/etc/init.d/S45cron start > /dev/null

			log "${TYPE}" "Restarting AppFS-Services."
			for i in /usr/local/init/S??* ; do
				# Ignore dangling symlinks (if any).
				[ ! -f "${i}" ] && continue
				if [ "$i" = "/usr/local/init/S99upgrade-firmware" ] || [ "$i" = "/usr/local/init/S02update-conf" ]; then
					continue
				fi
				${i} start
			done
			${BEROCONF} set root app-status running

			log "${TYPE}" "Restarting UserAppFS-Services."
                	/etc/init.d/S91userapp start > /dev/null
		fi
		;;
esac

if [ -f /tmp/reboot_after_update ]; then
	log ${TYPE} "Rebooting beroNet VoIP Gateway to complete the Update."
fi
sync
sleep 1
rm -f ${PIDFILE}
exit 0
